@@ -8,13 +8,14 @@ use deno_core::located_script_name;
 use deno_core::url::Url;
 use deno_core::JsRuntime;
 
-pub use esm_resolver::NodeEsmResolver;
+pub use esm_resolver::check_if_should_use_esm_loader;
+pub(crate) use esm_resolver::NodeEsmResolver;
 
 // TODO(bartlomieju): this needs to be bumped manually for
 // each release, a better mechanism is preferable, but it's a quick and dirty
 // solution to avoid printing `X-Deno-Warning` headers when the compat layer is
 // downloaded
-static STD_URL_STR: &str = "https://deno.land/std@0.112.0/";
+static STD_URL_STR: &str = "https://deno.land/std@0.114.0/";
 
 static SUPPORTED_MODULES: &[&str] = &[
   "assert",
@@ -62,9 +63,11 @@ static SUPPORTED_MODULES: &[&str] = &[
 ];
 
 lazy_static::lazy_static! {
-  static ref GLOBAL_URL_STR: String = format!("{}node/global.ts", STD_URL_STR);
+  static ref NODE_COMPAT_URL: String = std::env::var("DENO_NODE_COMPAT_URL").map(String::into).ok()
+    .unwrap_or_else(|| STD_URL_STR.to_string());
+  static ref GLOBAL_URL_STR: String = format!("{}node/global.ts", NODE_COMPAT_URL.as_str());
   pub(crate) static ref GLOBAL_URL: Url = Url::parse(&GLOBAL_URL_STR).unwrap();
-  static ref MODULE_URL_STR: String = format!("{}node/module.ts", STD_URL_STR);
+  static ref MODULE_URL_STR: String = format!("{}node/module.ts", NODE_COMPAT_URL.as_str());
   pub(crate) static ref MODULE_URL: Url = Url::parse(&MODULE_URL_STR).unwrap();
   static ref COMPAT_IMPORT_URL: Url = Url::parse("flags:compat").unwrap();
 }
@@ -76,41 +79,15 @@ pub(crate) fn get_node_imports() -> Vec<(Url, Vec<String>)> {
 
 fn try_resolve_builtin_module(specifier: &str) -> Option<Url> {
   if SUPPORTED_MODULES.contains(&specifier) {
-    let module_url = format!("{}node/{}.ts", STD_URL_STR, specifier);
+    let module_url =
+      format!("{}node/{}.ts", NODE_COMPAT_URL.as_str(), specifier);
     Some(Url::parse(&module_url).unwrap())
   } else {
     None
   }
 }
 
-pub async fn check_if_should_use_esm_loader(
-  js_runtime: &mut JsRuntime,
-  main_module: &str,
-) -> Result<bool, AnyError> {
-  // Decide if we're running with Node ESM loader or CJS loader.
-  let source_code = &format!(
-    r#"(async function checkIfEsm(main) {{
-      const {{ resolveMainPath, shouldUseESMLoader }} = await import("{}");
-      const resolvedMain = resolveMainPath(main);
-      const useESMLoader = shouldUseESMLoader(resolvedMain);
-      return useESMLoader;
-    }})('{}');"#,
-    MODULE_URL_STR.as_str(),
-    escape_for_single_quote_string(main_module),
-  );
-  let result =
-    js_runtime.execute_script(&located_script_name!(), source_code)?;
-  let use_esm_loader_global = js_runtime.resolve_value(result).await?;
-  let use_esm_loader = {
-    let scope = &mut js_runtime.handle_scope();
-    let use_esm_loader_local = use_esm_loader_global.get(scope);
-    use_esm_loader_local.boolean_value(scope)
-  };
-
-  Ok(use_esm_loader)
-}
-
-pub fn load_cjs_module(
+pub(crate) fn load_cjs_module(
   js_runtime: &mut JsRuntime,
   main_module: &str,
 ) -> Result<(), AnyError> {
