@@ -11,7 +11,6 @@ mod code_lens;
 mod completions;
 mod config;
 mod diagnostics;
-mod document_source;
 mod documents;
 pub(crate) mod language_server;
 mod lsp_custom;
@@ -21,7 +20,6 @@ mod performance;
 mod refactor;
 mod registries;
 mod semantic_tokens;
-mod sources;
 mod text;
 mod tsc;
 mod urls;
