@@ -30,11 +30,11 @@ mod resolver;
 mod source_maps;
 mod standalone;
 mod text_encoding;
-mod tokio_util;
 mod tools;
 mod tsc;
 mod unix_util;
 mod version;
+mod windows_util;
 
 use crate::file_fetcher::File;
 use crate::file_watcher::ResolutionResult;
@@ -60,6 +60,7 @@ use crate::fmt_errors::PrettyJsError;
 use crate::module_loader::CliModuleLoader;
 use crate::proc_state::ProcState;
 use crate::resolver::ImportMapResolver;
+use crate::resolver::JsxResolver;
 use crate::source_maps::apply_source_map;
 use crate::tools::installer::infer_name_from_url;
 use deno_ast::MediaType;
@@ -77,6 +78,7 @@ use deno_core::ModuleSpecifier;
 use deno_runtime::colors;
 use deno_runtime::ops::worker_host::CreateWebWorkerCb;
 use deno_runtime::permissions::Permissions;
+use deno_runtime::tokio_util::run_basic;
 use deno_runtime::web_worker::WebWorker;
 use deno_runtime::web_worker::WebWorkerOptions;
 use deno_runtime::worker::MainWorker;
@@ -203,6 +205,32 @@ pub fn create_main_worker(
 
   let create_web_worker_cb = create_web_worker_callback(ps.clone());
 
+  let maybe_storage_key = if let Some(location) = &ps.flags.location {
+    // if a location is set, then the ascii serialization of the location is
+    // used, unless the origin is opaque, and then no storage origin is set, as
+    // we can't expect the origin to be reproducible
+    let storage_origin = location.origin().ascii_serialization();
+    if storage_origin == "null" {
+      None
+    } else {
+      Some(storage_origin)
+    }
+  } else if let Some(config_file) = &ps.maybe_config_file {
+    // otherwise we will use the path to the config file
+    config_file.path.to_str().map(|s| s.to_string())
+  } else {
+    // otherwise we will use the path to the main module
+    Some(main_module.to_string())
+  };
+
+  let origin_storage_dir = maybe_storage_key.map(|key| {
+    ps.dir
+      .root
+      // TODO(@crowlKats): change to origin_data for 2.0
+      .join("location_data")
+      .join(checksum::gen(&[key.as_bytes()]))
+  });
+
   let options = WorkerOptions {
     bootstrap: BootstrapOptions {
       apply_source_maps: true,
@@ -230,14 +258,7 @@ pub fn create_main_worker(
     should_break_on_first_statement,
     module_loader,
     get_error_class_fn: Some(&crate::errors::get_error_class_name),
-    origin_storage_dir: ps.flags.location.clone().map(|loc| {
-      ps.dir
-        .root
-        .clone()
-        // TODO(@crowlKats): change to origin_data for 2.0
-        .join("location_data")
-        .join(checksum::gen(&[loc.to_string().as_bytes()]))
-    }),
+    origin_storage_dir,
     blob_store: ps.blob_store.clone(),
     broadcast_channel: ps.broadcast_channel.clone(),
     shared_array_buffer_store: Some(ps.shared_array_buffer_store.clone()),
@@ -448,14 +469,29 @@ async fn info_command(
       Permissions::allow_all(),
     );
     let maybe_locker = lockfile::as_maybe_locker(ps.lockfile.clone());
-    let maybe_resolver =
-      ps.maybe_import_map.as_ref().map(ImportMapResolver::new);
+    let maybe_import_map_resolver =
+      ps.maybe_import_map.clone().map(ImportMapResolver::new);
+    let maybe_jsx_resolver = ps
+      .maybe_config_file
+      .as_ref()
+      .map(|cf| {
+        cf.to_maybe_jsx_import_source_module()
+          .map(|im| JsxResolver::new(im, maybe_import_map_resolver.clone()))
+      })
+      .flatten();
+    let maybe_resolver = if maybe_jsx_resolver.is_some() {
+      maybe_jsx_resolver.as_ref().map(|jr| jr.as_resolver())
+    } else {
+      maybe_import_map_resolver
+        .as_ref()
+        .map(|im| im.as_resolver())
+    };
     let graph = deno_graph::create_graph(
       vec![specifier],
       false,
       None,
       &mut cache,
-      maybe_resolver.as_ref().map(|r| r.as_resolver()),
+      maybe_resolver,
       maybe_locker,
       None,
     )
@@ -548,7 +584,7 @@ async fn cache_command(
       Permissions::allow_all(),
     )
     .await?;
-    if let Some(graph_error) = ps.maybe_graph_error.lock().take() {
+    if let Some(graph_error) = ps.take_graph_error() {
       return Err(graph_error.into());
     }
   }
@@ -617,19 +653,35 @@ async fn create_graph_and_maybe_check(
     Permissions::allow_all(),
   );
   let maybe_locker = lockfile::as_maybe_locker(ps.lockfile.clone());
-  let maybe_imports = ps
+  let maybe_imports = if let Some(config_file) = &ps.maybe_config_file {
+    config_file.to_maybe_imports()?
+  } else {
+    None
+  };
+  let maybe_import_map_resolver =
+    ps.maybe_import_map.clone().map(ImportMapResolver::new);
+  let maybe_jsx_resolver = ps
     .maybe_config_file
     .as_ref()
-    .map(|cf| cf.to_maybe_imports())
+    .map(|cf| {
+      cf.to_maybe_jsx_import_source_module()
+        .map(|im| JsxResolver::new(im, maybe_import_map_resolver.clone()))
+    })
     .flatten();
-  let maybe_resolver = ps.maybe_import_map.as_ref().map(ImportMapResolver::new);
+  let maybe_resolver = if maybe_jsx_resolver.is_some() {
+    maybe_jsx_resolver.as_ref().map(|jr| jr.as_resolver())
+  } else {
+    maybe_import_map_resolver
+      .as_ref()
+      .map(|im| im.as_resolver())
+  };
   let graph = Arc::new(
     deno_graph::create_graph(
       vec![root],
       false,
       maybe_imports,
       &mut cache,
-      maybe_resolver.as_ref().map(|r| r.as_resolver()),
+      maybe_resolver,
       maybe_locker,
       None,
     )
@@ -945,19 +997,34 @@ async fn run_with_watch(flags: Flags, script: String) -> Result<(), AnyError> {
         Permissions::allow_all(),
       );
       let maybe_locker = lockfile::as_maybe_locker(ps.lockfile.clone());
-      let maybe_imports = ps
+      let maybe_imports = if let Some(config_file) = &ps.maybe_config_file {
+        config_file.to_maybe_imports()?
+      } else {
+        None
+      };
+      let maybe_import_map_resolver =
+        ps.maybe_import_map.clone().map(ImportMapResolver::new);
+      let maybe_jsx_resolver = ps
         .maybe_config_file
         .as_ref()
-        .map(|cf| cf.to_maybe_imports())
+        .map(|cf| {
+          cf.to_maybe_jsx_import_source_module()
+            .map(|im| JsxResolver::new(im, maybe_import_map_resolver.clone()))
+        })
         .flatten();
-      let maybe_resolver =
-        ps.maybe_import_map.as_ref().map(ImportMapResolver::new);
+      let maybe_resolver = if maybe_jsx_resolver.is_some() {
+        maybe_jsx_resolver.as_ref().map(|jr| jr.as_resolver())
+      } else {
+        maybe_import_map_resolver
+          .as_ref()
+          .map(|im| im.as_resolver())
+      };
       let graph = deno_graph::create_graph(
         vec![main_module.clone()],
         false,
         maybe_imports,
         &mut cache,
-        maybe_resolver.as_ref().map(|r| r.as_resolver()),
+        maybe_resolver,
         maybe_locker,
         None,
       )
@@ -1133,11 +1200,7 @@ async fn run_command(
     // this file.
     worker.execute_side_module(&compat::MODULE_URL).await?;
 
-    let use_esm_loader = compat::check_if_should_use_esm_loader(
-      &mut worker.js_runtime,
-      &main_module.to_file_path().unwrap().display().to_string(),
-    )
-    .await?;
+    let use_esm_loader = compat::check_if_should_use_esm_loader(&main_module)?;
 
     if use_esm_loader {
       // ES module execution in Node compatiblity mode
@@ -1364,14 +1427,16 @@ fn unwrap_or_exit<T>(result: Result<T, AnyError>) -> T {
 
 pub fn main() {
   setup_exit_process_panic_hook();
+
+  unix_util::raise_fd_limit();
+  windows_util::ensure_stdio_open();
   #[cfg(windows)]
   colors::enable_ansi(); // For Windows 10
-  unix_util::raise_fd_limit();
 
   let args: Vec<String> = env::args().collect();
   let standalone_res = match standalone::extract_standalone(args.clone()) {
     Ok(Some((metadata, bundle))) => {
-      tokio_util::run_basic(standalone::run(bundle, metadata))
+      run_basic(standalone::run(bundle, metadata))
     }
     Ok(None) => Ok(()),
     Err(err) => Err(err),
@@ -1399,5 +1464,5 @@ pub fn main() {
 
   logger::init(flags.log_level);
 
-  unwrap_or_exit(tokio_util::run_basic(get_subcommand(flags)));
+  unwrap_or_exit(run_basic(get_subcommand(flags)));
 }
