@@ -431,7 +431,7 @@ fn lsp_hover_asset() {
       "deno/virtualTextDocument",
       json!({
         "textDocument": {
-          "uri": "deno:/asset//lib.deno.shared_globals.d.ts"
+          "uri": "deno:asset/lib.deno.shared_globals.d.ts"
         }
       }),
     )
@@ -442,7 +442,7 @@ fn lsp_hover_asset() {
       "textDocument/hover",
       json!({
         "textDocument": {
-          "uri": "deno:/asset//lib.es2015.symbol.wellknown.d.ts"
+          "uri": "deno:asset/lib.es2015.symbol.wellknown.d.ts"
         },
         "position": {
           "line": 109,
@@ -919,11 +919,11 @@ fn lsp_hover_dependency() {
       "range": {
         "start": {
           "line": 0,
-          "character": 20
+          "character": 19
         },
         "end":{
           "line": 0,
-          "character": 61
+          "character": 62
         }
       }
     }))
@@ -953,11 +953,11 @@ fn lsp_hover_dependency() {
       "range": {
         "start": {
           "line": 3,
-          "character": 20
+          "character": 19
         },
         "end":{
           "line": 3,
-          "character": 66
+          "character": 67
         }
       }
     }))
@@ -987,11 +987,11 @@ fn lsp_hover_dependency() {
       "range": {
         "start": {
           "line": 4,
-          "character": 20
+          "character": 19
         },
         "end":{
           "line": 4,
-          "character": 56
+          "character": 57
         }
       }
     }))
@@ -1021,11 +1021,11 @@ fn lsp_hover_dependency() {
       "range": {
         "start": {
           "line": 5,
-          "character": 20
+          "character": 19
         },
         "end":{
           "line": 5,
-          "character": 131
+          "character": 132
         }
       }
     }))
@@ -1055,17 +1055,87 @@ fn lsp_hover_dependency() {
       "range": {
         "start": {
           "line": 6,
-          "character": 20
+          "character": 19
         },
         "end":{
           "line": 6,
-          "character": 32
+          "character": 33
         }
       }
     }))
   );
 }
 
+#[test]
+fn lsp_hover_typescript_types() {
+  let _g = http_server();
+  let mut client = init("initialize_params.json");
+  did_open(
+    &mut client,
+    json!({
+      "textDocument": {
+        "uri": "file:///a/file.ts",
+        "languageId": "typescript",
+        "version": 1,
+        "text": "import * as a from \"http://127.0.0.1:4545/xTypeScriptTypes.js\";\n\nconsole.log(a.foo);\n",
+      }
+    }),
+  );
+  let (maybe_res, maybe_err) = client
+    .write_request::<_, _, Value>(
+      "deno/cache",
+      json!({
+        "referrer": {
+          "uri": "file:///a/file.ts",
+        },
+        "uris": [
+          {
+            "uri": "http://127.0.0.1:4545/xTypeScriptTypes.js",
+          }
+        ],
+      }),
+    )
+    .unwrap();
+  assert!(maybe_err.is_none());
+  assert!(maybe_res.is_some());
+  let (maybe_res, maybe_err) = client
+    .write_request::<_, _, Value>(
+      "textDocument/hover",
+      json!({
+        "textDocument": {
+          "uri": "file:///a/file.ts"
+        },
+        "position": {
+          "line": 0,
+          "character": 24
+        }
+      }),
+    )
+    .unwrap();
+  assert!(maybe_res.is_some());
+  assert!(maybe_err.is_none());
+  assert_eq!(
+    json!(maybe_res.unwrap()),
+    json!({
+      "contents": {
+        "kind": "markdown",
+        "value": "**Resolved Dependency**\n\n**Code**: http&#8203;://127.0.0.1:4545/xTypeScriptTypes.js\n\n**Types**: http&#8203;://127.0.0.1:4545/xTypeScriptTypes.d.ts\n"
+      },
+      "range": {
+        "start": {
+          "line": 0,
+          "character": 19
+        },
+        "end": {
+          "line": 0,
+          "character": 62
+        }
+      }
+    })
+  );
+  shutdown(&mut client);
+}
+
 #[test]
 fn lsp_call_hierarchy() {
   let mut client = init("initialize_params.json");
@@ -1771,7 +1841,7 @@ fn lsp_code_lens_non_doc_nav_tree() {
       "deno/virtualTextDocument",
       json!({
         "textDocument": {
-          "uri": "deno:/asset//lib.deno.shared_globals.d.ts"
+          "uri": "deno:asset/lib.deno.shared_globals.d.ts"
         }
       }),
     )
@@ -1783,7 +1853,7 @@ fn lsp_code_lens_non_doc_nav_tree() {
       "textDocument/codeLens",
       json!({
         "textDocument": {
-          "uri": "deno:/asset//lib.deno.shared_globals.d.ts"
+          "uri": "deno:asset/lib.deno.shared_globals.d.ts"
         }
       }),
     )
@@ -1931,15 +2001,24 @@ fn lsp_signature_help() {
       "signatures": [
         {
           "label": "add(a: number, b: number): number",
-          "documentation": "Adds two numbers.",
+          "documentation": {
+            "kind": "markdown",
+            "value": "Adds two numbers."
+          },
           "parameters": [
             {
               "label": "a: number",
-              "documentation": "This is a first number."
+              "documentation": {
+                "kind": "markdown",
+                "value": "This is a first number."
+              }
             },
             {
               "label": "b: number",
-              "documentation": "This is a second number."
+              "documentation": {
+                "kind": "markdown",
+                "value": "This is a second number."
+              }
             }
           ]
         }
@@ -1995,15 +2074,24 @@ fn lsp_signature_help() {
       "signatures": [
         {
           "label": "add(a: number, b: number): number",
-          "documentation": "Adds two numbers.",
+          "documentation": {
+            "kind": "markdown",
+            "value": "Adds two numbers."
+          },
           "parameters": [
             {
               "label": "a: number",
-              "documentation": "This is a first number."
+              "documentation": {
+                "kind": "markdown",
+                "value": "This is a first number."
+              }
             },
             {
               "label": "b: number",
-              "documentation": "This is a second number."
+              "documentation": {
+                "kind": "markdown",
+                "value": "This is a second number."
+              }
             }
           ]
         }
@@ -2714,11 +2802,11 @@ fn lsp_cache_location() {
       "range": {
         "start": {
           "line": 0,
-          "character": 20
+          "character": 19
         },
         "end":{
           "line": 0,
-          "character": 61
+          "character": 62
         }
       }
     }))
@@ -3596,3 +3684,82 @@ fn lsp_lint_with_config() {
   }
   shutdown(&mut client);
 }
+
+#[test]
+fn lsp_jsx_import_source_pragma() {
+  let _g = http_server();
+  let mut client = init("initialize_params.json");
+  did_open(
+    &mut client,
+    json!({
+      "textDocument": {
+        "uri": "file:///a/file.tsx",
+        "languageId": "typescriptreact",
+        "version": 1,
+        "text":
+"/** @jsxImportSource http://localhost:4545/jsx */
+
+function A() {
+  return \"hello\";
+}
+
+export function B() {
+  return <A></A>;
+}
+",
+      }
+    }),
+  );
+  let (maybe_res, maybe_err) = client
+    .write_request::<_, _, Value>(
+      "deno/cache",
+      json!({
+        "referrer": {
+          "uri": "file:///a/file.tsx",
+        },
+        "uris": [
+          {
+            "uri": "http://127.0.0.1:4545/jsx/jsx-runtime",
+          }
+        ],
+      }),
+    )
+    .unwrap();
+  assert!(maybe_err.is_none());
+  assert!(maybe_res.is_some());
+  let (maybe_res, maybe_err) = client
+    .write_request::<_, _, Value>(
+      "textDocument/hover",
+      json!({
+        "textDocument": {
+          "uri": "file:///a/file.tsx"
+        },
+        "position": {
+          "line": 0,
+          "character": 25
+        }
+      }),
+    )
+    .unwrap();
+  assert!(maybe_err.is_none());
+  assert_eq!(
+    maybe_res,
+    Some(json!({
+      "contents": {
+        "kind": "markdown",
+        "value": "**Resolved Dependency**\n\n**Code**: http&#8203;://localhost:4545/jsx/jsx-runtime\n",
+      },
+      "range": {
+        "start": {
+          "line": 0,
+          "character": 21
+        },
+        "end": {
+          "line": 0,
+          "character": 46
+        }
+      }
+    }))
+  );
+  shutdown(&mut client);
+}
