@@ -6,7 +6,7 @@ import {
   unitTest,
 } from "./test_util.ts";
 
-unitTest(function eventInitializedWithType(): void {
+unitTest(function eventInitializedWithType() {
   const type = "click";
   const event = new Event(type);
 
@@ -18,7 +18,7 @@ unitTest(function eventInitializedWithType(): void {
   assertEquals(event.cancelable, false);
 });
 
-unitTest(function eventInitializedWithTypeAndDict(): void {
+unitTest(function eventInitializedWithTypeAndDict() {
   const init = "submit";
   const eventInit = { bubbles: true, cancelable: true } as EventInit;
   const event = new Event(init, eventInit);
@@ -31,7 +31,7 @@ unitTest(function eventInitializedWithTypeAndDict(): void {
   assertEquals(event.cancelable, true);
 });
 
-unitTest(function eventComposedPathSuccess(): void {
+unitTest(function eventComposedPathSuccess() {
   const type = "click";
   const event = new Event(type);
   const composedPath = event.composedPath();
@@ -39,7 +39,7 @@ unitTest(function eventComposedPathSuccess(): void {
   assertEquals(composedPath, []);
 });
 
-unitTest(function eventStopPropagationSuccess(): void {
+unitTest(function eventStopPropagationSuccess() {
   const type = "click";
   const event = new Event(type);
 
@@ -48,7 +48,7 @@ unitTest(function eventStopPropagationSuccess(): void {
   assertEquals(event.cancelBubble, true);
 });
 
-unitTest(function eventStopImmediatePropagationSuccess(): void {
+unitTest(function eventStopImmediatePropagationSuccess() {
   const type = "click";
   const event = new Event(type);
 
@@ -57,7 +57,7 @@ unitTest(function eventStopImmediatePropagationSuccess(): void {
   assertEquals(event.cancelBubble, true);
 });
 
-unitTest(function eventPreventDefaultSuccess(): void {
+unitTest(function eventPreventDefaultSuccess() {
   const type = "click";
   const event = new Event(type);
 
@@ -72,7 +72,7 @@ unitTest(function eventPreventDefaultSuccess(): void {
   assertEquals(cancelableEvent.defaultPrevented, true);
 });
 
-unitTest(function eventInitializedWithNonStringType(): void {
+unitTest(function eventInitializedWithNonStringType() {
   // deno-lint-ignore no-explicit-any
   const type: any = undefined;
   const event = new Event(type);
@@ -86,7 +86,7 @@ unitTest(function eventInitializedWithNonStringType(): void {
 });
 
 // ref https://github.com/web-platform-tests/wpt/blob/master/dom/events/Event-isTrusted.any.js
-unitTest(function eventIsTrusted(): void {
+unitTest(function eventIsTrusted() {
   const desc1 = Object.getOwnPropertyDescriptor(new Event("x"), "isTrusted");
   assert(desc1);
   assertEquals(typeof desc1.get, "function");
@@ -98,7 +98,7 @@ unitTest(function eventIsTrusted(): void {
   assertEquals(desc1!.get, desc2!.get);
 });
 
-unitTest(function eventInspectOutput(): void {
+unitTest(function eventInspectOutput() {
   // deno-lint-ignore no-explicit-any
   const cases: Array<[any, (event: any) => string]> = [
     [
@@ -133,7 +133,7 @@ unitTest(function eventInspectOutput(): void {
   }
 });
 
-unitTest(function inspectEvent(): void {
+unitTest(function inspectEvent() {
   // has a customInspect implementation that previously would throw on a getter
   assertEquals(
     Deno.inspect(Event.prototype),
